/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic
 * Semiconductor ASA.Terms and conditions of usage are described in detail
 * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 249 $
 */
 
#ifndef _REGS_COMMON_H
#define _REGS_COMMON_H

/*lint -emacro(717,BITFIELD_WRITE) "Suppress do {...} while(false)" */
/*lint -emacro(778,ilog2) "Suppress Constant expression evaluates to 0" */
/*lint -emacro(506,ilog2) "Constant value Boolean" */

/*lint ++flb "Enter library region" */

/** @file
* @brief Helper macros for reading and writing bitfield values.
*
*
* @defgroup lib_bitfield_macros Helper macros for reading and writing bitfield values.
* @{
* @ingroup nrf_libs
* @brief Helper macros for reading and writing bitfield values.
* 
* nRF device peripheral header files follow a certain register bitfield naming convention. For reading and writing individual bitfields within 32-bit registers we provide few macros to make code mode readable.
* Register bitfield mask:      PERIPHERALNAME_REGISTERNAME_bitfield_Msk
* Register bitfield position:  PERIPHERALNAME_REGISTERNAME_bitfield_Pos
* Register bitfield values:    PERIPHERALNAME_REGISTERNAME_bitfield_VALUENAME
* 
* For example GPIOTE peripheral has a register named CONFIG which has a bitfields named POLARITY, PSEL and MODE. 
* Each of these bitfields have been defined in the following way:
*
* define GPIOTE_CONFIG_POLARITY_Pos (16UL)
* define GPIOTE_CONFIG_POLARITY_Msk (0x3UL << GPIOTE_CONFIG_POLARITY_Pos)
* define GPIOTE_CONFIG_POLARITY_LoToHi (0x01UL)
* define GPIOTE_CONFIG_POLARITY_HiToLo (0x02UL)
* define GPIOTE_CONFIG_POLARITY_Toggle (0x03UL)
*
* If you want to set all of the different fields at once you could write
* @code
* NRF_GPIOTE->CONFIG = (GPIOTE_CONFIG_POLARITY_HiToLo << GPIOTE_CONFIG_POLARITY_Pos)
*                    | (11UL                          << GPIOTE_CONFIG_PSEL_Pos)
*                    | (GPIOTE_CONFIG_MODE_Task       << GPIOTE_CONFIG_MODE_Pos);
* @endcode
*
* @note Putting "UL" after numerical values will ensure that the resulting type is uint32_t.
*
* If you want to read a single bitfield value from a register you can do this like this:
*
* @code
* uint32_t current_polarity = BITFIELD_READ(NRF_GPIOTE->CONFIG, GPIOTE_CONFIG_POLARITY_Msk);
* @endcode
*
* Which is equal to this:
*
* @code
* uint32_t current_polarity = (NRF_GPIOTE->CONFIG & GPIOTE_CONFIG_POLARITY_Msk) >> GPIOTE_CONFIG_POLARITY_Pos;
* @endcode
*
* Writing bitfield values follows the same convention:
*
* @code
* BITFIELD_WRITE(NRF_GPIOTE->CONFIG, GPIOTE_CONFIG_POLARITY_Msk, GPIOTE_CONFIG_POLARITY_LoToHi);
* @endcode
*
* @note In cases where the bitfield mask is a variable, BITFIELD_WRITE will generate a lot of overhead code and it is recommended to use @ref BITFIELD_WRITE_POS instead.
*
* Which is equal to this:
* @code
* NRF_GPIOTE->CONFIG = (NRF_GPIOTE->CONFIG & ~(GPIOTE_CONFIG_POLARITY_Msk)) | (GPIOTE_CONFIG_POLARITY_LoToHi << GPIOTE_CONFIG_POLARITY_Pos);
* @endcode
*
* @code
* BITFIELD_WRITE_POS(NRF_GPIOTE->CONFIG, GPIOTE_CONFIG_POLARITY_Msk, GPIOTE_CONFIG_POLARITY_Pos, GPIOTE_CONFIG_POLARITY_LoToHi);
* @endcode
*
*/

/** Inverse base-2 logarithm
*/
#define ilog2(n)                          \
 (                                        \
                 (n) & (1UL << 31) ? 31 : \
                 (n) & (1UL << 30) ? 30 : \
                 (n) & (1UL << 29) ? 29 : \
                 (n) & (1UL << 28) ? 28 : \
                 (n) & (1UL << 27) ? 27 : \
                 (n) & (1UL << 26) ? 26 : \
                 (n) & (1UL << 25) ? 25 : \
                 (n) & (1UL << 24) ? 24 : \
                 (n) & (1UL << 23) ? 23 : \
                 (n) & (1UL << 22) ? 22 : \
                 (n) & (1UL << 21) ? 21 : \
                 (n) & (1UL << 20) ? 20 : \
                 (n) & (1UL << 19) ? 19 : \
                 (n) & (1UL << 18) ? 18 : \
                 (n) & (1UL << 17) ? 17 : \
                 (n) & (1UL << 16) ? 16 : \
                 (n) & (1UL << 15) ? 15 : \
                 (n) & (1UL << 14) ? 14 : \
                 (n) & (1UL << 13) ? 13 : \
                 (n) & (1UL << 12) ? 12 : \
                 (n) & (1UL << 11) ? 11 : \
                 (n) & (1UL << 10) ? 10 : \
                 (n) & (1UL <<  9) ?  9 : \
                 (n) & (1UL <<  8) ?  8 : \
                 (n) & (1UL <<  7) ?  7 : \
                 (n) & (1UL <<  6) ?  6 : \
                 (n) & (1UL <<  5) ?  5 : \
                 (n) & (1UL <<  4) ?  4 : \
                 (n) & (1UL <<  3) ?  3 : \
                 (n) & (1UL <<  2) ?  2 : \
                 (n) & (1UL <<  1) ?  1 : \
                 (n) & (1UL <<  0) ?  0 : \
                 0      \
 )                                             

#define BITCOUNT_X(x) ((x) - (((x)>>1)&0x77777777) - (((x)>>2)&0x33333333) - (((x)>>3)&0x11111111)) //!< Helper macro for calculating number of set bits in a value
#define COUNT_BITS_SET(x)   (((BITCOUNT_X(x)+(BITCOUNT_X(x)>>4)) & 0x0F0F0F0F) % 255) //!< Calculates number of set bits in a value

//#define CREATE_MASK(bitfield_position, bitfield_bitcount) (((1ULL << (bitfield_bitcount)) - 1) << bitfield_position) //!< Creates a bit mask based on number of bits and bitfield position
//#define VALUEMASK(bitfield_mask) ((bitfield_mask) >> POS(bitfield_mask)) //!< Returns a bit mask for a value
//#define MASK(bitfield_mask) (bitfield_mask) //!< Returns a bit mask for a bit field shifted to the correct position in a register
//#define BITS(bitfield_mask) (COUNT_BITS_SET(bitfield_mask)) //!< Returns number of bits in a bit field
//#define SHIFT(bitfield_mask, bitfield_value) (bitfield_value << POS(bitfield_mask))

#define POS(bitfield_mask)  (ilog2(bitfield_mask & ~(bitfield_mask - 1))) //!< Returns the position of the least significant bit in a bitfield mask in a register

#define BITFIELD_READ(_register, _bitfield_mask) (((_register) & (_bitfield_mask)) >> POS(_bitfield_mask)) //!< Returns a value of a bitfield in a register
#define BITFIELD_VALUE(_bitfield_mask, _bitfield_value) ((_bitfield_value) << POS(_bitfield_mask)) //!< Returns a bit field value shifted to the correct position in a register
#define BITFIELD_WRITE(_register, _bitfield_mask, _bitfield_value) do { (_register) = ((_register) & ~(_bitfield_mask)) | (BITFIELD_VALUE(_bitfield_mask, _bitfield_value)); } while (false) //!< Writes a single bit field value to a register
#define BITFIELD_WRITE_POS(_register, _bitfield_mask, _bitfield_pos, _bitfield_value) do { (_register) = ((_register) & ~(_bitfield_mask)) | ((_bitfield_value) << (_bitfield_pos)); } while (false) //!< Writes a single bit field value to a register

/*lint --flb "Leave library region" */

/**
 *@}
 **/
 
#endif
