/* Copyright (c) 2011 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 7302 $
 */


/**
 * @file
 * @brief Gazell Link Layer.
 *
*/

#ifndef NRF_GZLL_H__
#define NRF_GZLL_H__

#include "stdbool.h"
#include "nrf.h"


/**
* @defgroup gzll_api Gazell Link Layer API
* @{
* @ingroup gzll
* @brief This is where the description for the Gazell link layer API should go.
*/



 /**
 * @enum NRF_GZLL_tx_power_t
 * @brief Enumerator used for selecting the transmit (TX) power.
 */
typedef enum
{
  NRF_GZLL_TX_POWER_4_DBM,          ///<  4 dBm transmit power.
  NRF_GZLL_TX_POWER_0_DBM,          ///<  0 dBm transmit power.
  NRF_GZLL_TX_POWER_N4_DBM,         ///< -4 dBm transmit power.
  NRF_GZLL_TX_POWER_N8_DBM,         ///< -8 dBm transmit power.
  NRF_GZLL_TX_POWER_N12_DBM,        ///< -12 dBm transmit power.
  NRF_GZLL_TX_POWER_N16_DBM,        ///< -16 dBm transmit power.
  NRF_GZLL_TX_POWER_N20_DBM         ///< -20 dBm transmit power.
} NRF_GZLL_tx_power_t;


/**
 * @enum NRF_GZLL_datarate_t
 * @brief Enumerator used for
 selecting the radio datarate.
 */
typedef enum
{
  NRF_GZLL_DATARATE_250KBIT,    ///<  250 Kbps datarate.
  NRF_GZLL_DATARATE_1MBIT,      ///<  1 Mbps datarate.
  NRF_GZLL_DATARATE_2MBIT       ///<  2 Mbps datarate.
} NRF_GZLL_datarate_t;


/**
 * @enum NRF_GZLL_base_address_length_t
 * @brief Enumerator used for selecting Gazell base address length which is 
 * appended after the (address) to make the full address.
 */
typedef enum
{
  NRF_GZLL_BASE_ADDRESS_LENGTH_2B = 2, ///< On air address equals base address plus 1 byte prefix
  NRF_GZLL_BASE_ADDRESS_LENGTH_3B,     ///< On air address equals base address plus 1 byte prefix
  NRF_GZLL_BASE_ADDRESS_LENGTH_4B,     ///< On air address equals base address plus 1 byte prefix
} NRF_GZLL_base_address_length_t;


 /**
 * @enum NRF_GZLL_mode_t
 * @brief Enumerator used for selecting Gazell mode.
 */
typedef enum
{
  NRF_GZLL_MODE_DEVICE,       ///< Device mode
  NRF_GZLL_MODE_HOST,         ///< Host mode.
  NRF_GZLL_NUMBER_OF_MODES    ///< Number of GZLL modes
} NRF_GZLL_mode_t;


 /**
 * @enum NRF_GZLL_exitcode_t
 * @brief Enumerator used for exit codes for Gazell API functions
 */
typedef enum
{
  NRF_GZLL_EXITCODE_SUCCESS,
  NRF_GZLL_EXITCODE_CHANNEL_TABLE_INVALID,
  NRF_GZLL_EXITCODE_TX_QUEUE_FULL,
  NRF_GZLL_EXITCODE_RX_QUEUE_EMPTY,
  NRF_GZLL_EXITCODE_PAYLOAD_LENGTH_INVALID,
  NRF_GZLL_EXITCODE_PIPE_NUMBER_INVALID,
  NRF_GZLL_EXITCODE_NUM_PIPES_INVALID,
  NRF_GZLL_EXITCODE_DATARATE_INVALID,
  NRF_GZLL_EXITCODE_CRC_LENGTH_INVALID,
  NRF_GZLL_EXITCODE_CRC_INITIAL_VALUE_INVALID,
  NRF_GZLL_EXITCODE_CRC_POLYNOMIAL_INVALID,
  NRF_GZLL_EXITCODE_UNKNOWN_ERROR,
  NRF_GZLL_EXITCODE_NUMBER_OF_EXIT_CODES
} NRF_GZLL_exitcode_t;


/**
 * @name General API functions
 * @{
 */



/**
 * @brief Function for initializing the Gazell Link Layer.
 *
 * @param mode The mode to initialize Gazell in. @sa NRF_GZLL_mode_t.
 * 
 * @return The pointer to the data structure holding the status of the FIFO library.
 * NULL is returned if the FIFO library was not successfully initialized.
 *
 */
void NRF_GZLL_init(NRF_GZLL_mode_t mode);


/*
 * @brief Function for passing new parameters to the Gazell Link Layer.
 *
 * New parameters can be passed during any mode and state of the Gazell 
 * Link Layer, but the parameters will not nececarilly apply immediately.
 *
 * When the new parameters have applied a NRF_GZLL_NOTIFICATION_PARAMS_UPDATED
 * will be sent to the application via the notification callback handler.
 *
 * @param params is a pointer to the data structure holding the new
 * configuration parameters to be used. The new parameters are copied when
 * calling this function, meaning that the parameters do not need to be
 * preserved outside this function. 
 * However, only a shallow copy is being made, so any data pointed to inside 
 * the parameter structure, such as the channel table, would have to be
 * preserved after calling this function.
 *
 *
//void NRF_GZLL_set_params(const NRF_GZLL_params_t* params);
 */


/**
 * @brief Function for enabling the Gazell Link Layer.
 *
 * When enabled the behaviour described for the current
 * Gazell Link Layer mode will apply. 
 *
 * @sa NRF_GZLL_set_mode() 
 *
 */
void NRF_GZLL_enable(void);

/**
 * @brief Function for disabling the Gazell Link Layer.
 *
 * When calling this function the Gazell Link Layer will be disabled 
 * immediately. However, if there are any pending notifications to be sent
 * to the application these will still be sent.
 *
 */
void NRF_GZLL_disable(void);

/** @} */


/******************************************************************************/
/** @name Packet transmission and receiving functions
  * @{ */
/******************************************************************************/

/**
 * @brief Push a packet to the tail of the TX buffer. 
 *
 * In Device mode, the packet will be added. 
 * In Host mode, the payload will be piggybacked onto an acknowledgement. 
 *
 * @param payload is a pointer to the payload. 
 * @param payload_length is the number of bytes of the payload to transmit. 
 * @param pipe is the pipe for which to add the payload.
 *
 * @retval 0 if the packet was successfully added to the transmit queue.
 * @retval 1 if unsuccessful, check NRF_GZLL_ERROR_CODE for more information. 
 */
bool NRF_GZLL_push_to_tx_queue(uint32_t pipe, uint8_t * payload, uint32_t payload_length);

/**
 * @brief Pop a packet from the head of the RX buffer. 
 *
 * @param payload is a pointer to copy the payload to. 
 * @param payload_length is a pointer to copy the payload length to. The payload length is given in bytes.
 * @param pipe is the pipe for which to add the payload.
 *
 * @returns true if packet successful.
 */
bool NRF_GZLL_pop_from_rx_queue(uint32_t pipe, uint8_t * payload, uint32_t* payload_length);

/**
 * @brief Get the number of elements in the TX queue on a specific pipe.
 *
 * @param pipe is the pipe for which to check.
 * @return number of elements.
 */
uint32_t NRF_GZLL_get_tx_queue_count(uint32_t pipe);

/**
 * @brief Get the number of elements in the RX queue on a specific 
 pipe.
 *
 * @param pipe is the pipe for which to check.
 * @return number of elements.
 * @return number of elements.
 */
uint8_t NRF_GZLL_get_rx_queue_count(uint32_t pipe);

/**
 * @brief Flush the tx queue for a specific pipe.
 *
 * Delete all the packets and free the memory of the queue for a specific pipe.
 *
 * @param pipe is the pipe for which to flush.
 */
void NRF_GZLL_flush_tx_queue(uint32_t pipe);

void NRF_GZLL_flush_rx_queue(uint32_t pipe);




/** @name Callback functions: Device mode only
@{
*/
//lint -save -e522 These callback functions can lack side effects.


/** 
 * @brief Ack received callback (Device mode only). 
 * 
 * This callback is made when the Device receives an acknowledgement. @sa NRF_GZLL_ack_payload_received.
 *
 * @param pipe is the pipe on which an ACK packet was received.
 */
__weak void NRF_GZLL_DEVICE_ack_received(uint32_t pipe);

/** 
 * @brief Ack payload received callback (Device mode only). 
 * 
 * This callback is made when a packet receives an acknowledgment with a payload which is now available from the FIFO. The user 
 * should pop the receive buffer for the relevant pipe using NRF_GZLL_pop_from_rx_queue and free the memory for the packet. 
 *
 * @param pipe is the pipe on which an ACK packet with a payload was received.
 */
__weak void NRF_GZLL_DEVICE_ack_payload_received(uint32_t pipe);

/** 
 * @brief Transmission failed callback. 
 * 
 * This callback is made when a packet does not receive acknowledgment after
 * NRF_GZLL_MAX_RETRIES is reached. 
 *
 * @param pipe is the pipe on which the transmission failed.
 */
__weak void NRF_GZLL_transmission_failed(uint32_t pipe);

/** @} */

/** @name Callback functions: Host mode only
@{
*/

/** 
 * @brief Data packet received callback (Host mode only). 
 * 
 * This callback is made when a Host receives a data packet from a Device. 
 *
 * @param pipe is the pipe on which the data packet was received.
 */
__weak void NRF_GZLL_HOST_data_packet_received(uint32_t pipe);


/** 
 * @brief ACK successful callback (Host mode only). 
 * 
 * This callback is made to the Host after sending an ACK and receiving
 * a new packet from the Device. 
 * Note that the Device may have reached the maximum number of attempts
 * and is attempting to send a new packet.
 *
 * @param pipe is the pipe on which an ACK packet was sent.
 */
__weak void NRF_GZLL_HOST_ack_successful(uint32_t pipe);


/** @} */

/** @name Callback functions: All modes
@{
*/

/** 
 * @brief Function callback when Gazell is disabled. 
 * 
 * This callback is made sometime after the user calls NRF_GZLL_disable() and Gazell has finished 
 * and remaining communications and is in a disabled state. Calls to any set functions may be made
 * now and will be in effect when Gazell is enabled. 
 *
 */
__weak void NRF_GZLL_disabled_callback(void);

/** @} */

//line -restore


/** @name Configuration functions
@{
*/

/**
 * @brief Function for changing the mode of the Gazell Link Layer.
 *
 * The new mode setting will not necessarily apply immediately.
 *
 * When the new mode setting has applied and the new mode is different from the
 * previous mode a NRF_GZLL_NOTIFICATION_MODE_CHANGED notification will be sent 
 * to the application via the notification callback handler.
 *
 * @param mode specifies the new Gazell Link Layer mode to be used. See 
 * NRF_GZLL_mode_t for a list of valid modes. 
 *
 */
void NRF_GZLL_set_mode(NRF_GZLL_mode_t mode);

NRF_GZLL_mode_t NRF_GZLL_get_mode(void);


/**
* @brief Set the table of Radio Frequency (RF) channels to be used by 
* Gazell.
* 
* The valid channels are in the range 0 << channel < 125, where the 
* actual frequency in MHz is given by  2400+ channel.
* Gazell should be disabled in order to change the channel table, and
* will apply after Gazell is enabled. @sa NRF_GZLL_disable, NRF_GZLL_enable.
*
* @param new_channel_table Pointer to the channel table. 
* @param size specifies the size of the channel table.
*
* @retval true If the channel was set correctly.
* @retval false if Gazell is enabled, or the parameters are invalid.
*/
bool NRF_GZLL_set_channel_table(uint8_t* new_channel_table, uint32_t size);

/**
* @brief Get a copy of the current channel table. 
*
* The user is required to supply a pointer to an array of sufficient size. 
*
* @param channel_table_ptr Pointer to the channel table. 
* @param size specifies the size of the channel table.

* @retval false if Gazell is enabled.
*/
bool NRF_GZLL_get_channel_table(uint8_t* channel_table_ptr, uint32_t size);

/**
* @brief Get the size of the current channel table. 
*
* @returns Size of the channel table.
*
*/
uint32_t NRF_GZLL_get_channel_table_size(void);

/**
* @brief Set the address for a specific pipe.
*
* The on-air address for each pipe is composed of 3-5 bytes.
*
* Pipes 1 to 7 must have the same address except for the first byte. 
* If this function is called to change the address on pipes 1 through to 7, 
* it will change the common part of the address. 
*
* @param pipe The pipe that the address should apply to. 
* @param address is the address byte.
*
* @returns false if Gazell is enabled, or if the pipe is invalid, or if the length is invalid.
*/
bool NRF_GZLL_set_address(uint32_t pipe, uint8_t address);

uint8_t NRF_GZLL_get_address(uint32_t pipe);

bool NRF_GZLL_set_base_address_0(NRF_GZLL_base_address_length_t length, uint8_t* base_address);
bool NRF_GZLL_set_base_address_1(NRF_GZLL_base_address_length_t length, uint8_t* base_address);

bool NRF_GZLL_get_base_address_0(NRF_GZLL_base_address_length_t max_length, uint8_t* base_address);
bool NRF_GZLL_get_base_address_1(NRF_GZLL_base_address_length_t max_length, uint8_t* base_address);








/** @} */
/** @} */

#endif
