 /* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
  *
  * The information contained herein is confidential property of Nordic
  * Semiconductor ASA.Terms and conditions of usage are described in detail
  * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
  *
  * Licensees are granted free, non-transferable use of the information. NO
  * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
  * the file.
  *
  */

#ifndef I2C_MASTER_H
#define I2C_MASTER_H

/*lint ++flb "Enter library region */

#include <stdbool.h>
#include <stdint.h>

/** @file
* @brief Software controlled I2C Master driver.
*
*
* @defgroup lib_driver_i2c_master Software controlled I2C Master driver.
* @{
* @ingroup nrf_libs
* @brief Software controlled I2C Master driver.
*
* Supported features:
* - Repeated start
* - No multi-master
* - Only 7-bit addressing
* - Supports clock stretching (with optional SMBus style slave timeout)
* - Tries to handle slaves stuck in the middle of transfer
*/

#define I2C_READ_BIT 0x01 //!< If this bit is set in the address field, transfer direction is from slave to master.

#define I2C_ISSUE_START (bool)true //!< Parameter for @ref i2c_master_transfer
#define I2C_ISSUE_STOP (bool)true  //!< Parameter for @ref i2c_master_transfer

#define I2C_DONT_ISSUE_START (bool)false //!< Parameter for @ref i2c_master_transfer
#define I2C_DONT_ISSUE_STOP (bool)false //!< Parameter for @ref i2c_master_transfer

/**
 * Initializes I2C bus IO pins and checks the bus is operational.
 *
 * Both pins are configured as Standard-0, No-drive-1 (open drain).
 * If a global defines m_i2c_clock_pin_number or m_i2c_data_pin_number are used to set pin numbers during compilation,
 * clock_pin_number and data_pin_number are not used.
 *
 * @param clock_pin_number Pin number for SCL.
 * @param data_pin_number Pin number for SDA.
 * @return
 * @retval true I2C bus is clear for transfers.
 * @retval false I2C bus is stuck.
 */
bool i2c_master_init(uint8_t clock_pin_number, uint8_t data_pin_number);

/**
 * Transfers data over I2C bus.
 * 
 * If I2C master detects even one NACK from the slave or timeout occurs, STOP condition is issued
 * and the function returns false.
 * Bit 0 (@ref I2C_READ_BIT) in the address parameter controls transfer direction; 
 * - If 1, master reads data_length number of bytes from the slave
 * - If 0, master writes data_length number of bytes to the slave.
 *
 * @note Make sure at least data_length number of bytes is allocated in data if I2C_READ_BIT is set.
 * @note @ref I2C_ISSUE_STOP
 *
 * @param address Data transfer direction (LSB) / Slave address (7 MSBs).
 * @param data Pointer to data.
 * @param data_length Number of bytes to transfer.
 * @param issue_start_condition If @ref I2C_ISSUE_START, START condition is issued before transferring data. If @ref I2C_DONT_ISSUE_START, START condition is not issued before transferring data.
 * @param issue_stop_condition If @ref I2C_ISSUE_STOP, STOP condition is issued before exiting function. If @ref I2C_DONT_ISSUE_STOP, STOP condition is not issued before exiting function. If transfer failed for any reason, STOP condition will be issued in any case.
 * @return
 * @retval true Data transfer succeeded without errors.
 * @retval false Data transfer failed.
 */
bool i2c_master_transfer(uint8_t address, uint8_t *data, uint8_t data_length, bool issue_start_condition, bool issue_stop_condition);

/**
 *@}
 **/

/*lint --flb "Leave library region" */ 
#endif
