
/*
	Gazelle protocol config and API layer.
	2012 Even Brenden
*/

#include "gazell.h"

#ifdef USE_GZLL

	//#define PRINT

	/* RX def. */
	static uint8_t rx_packet[PACKET_SIZE_TOTAL];			///< The one that the radio writes to. Volatile.
	static uint8_t received_packet_1[PACKET_SIZE_TOTAL];	///< The ones that buffers up incoming packets.
	static uint8_t received_packet_2[PACKET_SIZE_TOTAL];
	static uint8_t no_packets_received;						///< Number of packets buffered up.
	static bool rx_packet_ready;							///< Flag to indicate that there are unread packets.
	static uint32_t rx_packet_payload_length;

	/* TX def. */
	static uint8_t tx_packet[PACKET_SIZE_TOTAL];
	static uint8_t building_packet[PACKET_SIZE_TOTAL];
	static uint32_t sp_bit_ptr;

	/* Gazell config. */
	static uint32_t channel_table_size = MY_GZLL_PARAMS_CHANNEL_TABLE_SIZE;                         ///< Number of different Radio frequency (RF) channels to cycle through.
	static uint8_t channel_table[MY_GZLL_PARAMS_CHANNEL_TABLE_SIZE] = MY_GZLL_PARAMS_CHANNEL_TABLE; ///< Number of RF channels to cycle through.
	static uint8_t rxtx_pipe = 0;                                    								///< Pipe that the host and device is on.
	static uint8_t address = MY_GZLL_PARAMS_ADDRESS_PIPE_0;     									///< Address of rxtx_pipe to host.
	static uint8_t base_address[4] = MY_GZLL_PARAMS_BASE_ADDRESS_0;       							///< Base address of rxtx_pipe to host.

	void gazell_init() {

	   	/* RX inits. */
		memset(rx_packet, 0L, PACKET_SIZE_TOTAL);
		memset(received_packet_1, 0L, PACKET_SIZE_TOTAL);
		memset(received_packet_2, 0L, PACKET_SIZE_TOTAL);
		no_packets_received=0U;
		rx_packet_ready=false;

	   	/* TX inits. */
		memset(tx_packet, 0L, PACKET_SIZE_TOTAL);
		memset(building_packet, 0L, PACKET_SIZE_TOTAL);
		sp_bit_ptr = 0;

		#ifdef IS_DEVICE
		    NRF_GZLL_init(NRF_GZLL_MODE_DEVICE);
		#endif
		#ifdef IS_HOST
		    NRF_GZLL_init(NRF_GZLL_MODE_HOST);
		#endif

		/* Put an empty packet in TX cue to initialize stream at host (piggyback first ACK). */
		#ifdef IS_HOST
			transmit();
		#endif

	    (void)NRF_GZLL_set_channel_table(channel_table,channel_table_size);
	    (void)NRF_GZLL_set_address(rxtx_pipe, address);
	    (void)NRF_GZLL_set_base_address_0 (NRF_GZLL_BASE_ADDRESS_LENGTH_4B, base_address);

	    NRF_GZLL_enable();
	}

	#ifdef IS_HOST
	void NRF_GZLL_HOST_data_packet_received(uint32_t pipe, bool packet_popped_from_tx_queue) {

		#ifdef PROFILE
			NRF_GPIO->OUTSET = (RX_PIN);
		#endif

	    (void)NRF_GZLL_pop_from_rx_queue(rxtx_pipe, rx_packet, &rx_packet_payload_length);

		rx_packet_ready=true;

		#ifdef PROFILE
			NRF_GPIO->OUTCLR = (RX_PIN);
		#endif
	}
	#endif

	#ifdef IS_DEVICE
		void NRF_GZLL_DEVICE_ack_received(uint32_t pipe, bool has_payload) {

			if(has_payload) {

				#ifdef PROFILE
					NRF_GPIO->OUTSET = (RX_PIN);
				#endif

			    (void)NRF_GZLL_pop_from_rx_queue(rxtx_pipe, rx_packet, &rx_packet_payload_length);

				rx_packet_ready=true;

				#ifdef PROFILE
					NRF_GPIO->OUTCLR = (RX_PIN);
				#endif
			}
		}
	#endif

	/* Transmit. */
	void transmit() {

		#ifdef PROFILE
			NRF_GPIO->OUTSET = (TX_PIN);
		#endif

		(void)NRF_GZLL_push_to_tx_queue(rxtx_pipe, tx_packet, PACKET_SIZE_TOTAL);

		#ifdef PROFILE
			NRF_GPIO->OUTCLR = (TX_PIN);
		#endif
	}

	/* Equal to set_tx_packet(), but with double buffers in case a (well, two) packet(s) comes in while another is being decoded. */
	void buffer_up_rx_packet() {

		uint32_t n;

		if (no_packets_received==0U) {
			for(n=0;n<PACKET_SIZE_TOTAL;n++) {
				received_packet_1[n] = rx_packet[n];
			}
			no_packets_received=1U;
		} else if (no_packets_received==1U) {
			for(n=0;n<PACKET_SIZE_TOTAL;n++) {
				received_packet_2[n] = rx_packet[n];
			}
			no_packets_received=2U;
		} else {
			/* Discard incoming packet. Or discard oldest packet? */
		}
	}

	/* Check flag from main(). */
	bool RX_next_packet_is_ready() {
		return rx_packet_ready;
	}

	/* Fetch newly received	packet. */
	void fetch_new_samples(Q16 *samples, Q16 *lpc, uint32_t *sbits) {

		/* Should this one block? And will it ever with the current main loop? */
		while(no_packets_received==0);

		/* Unpack and write to *samples. */
		if(no_packets_received==1U) {

			unpack_packet(received_packet_1, &samples[0], &lpc[0], sbits);
			no_packets_received=0U;
		} else if(no_packets_received==2U) {

			unpack_packet(received_packet_2, &samples[0], &lpc[0], sbits);
			no_packets_received=1U;
		}
		rx_packet_ready=false;
	}

	/* Extract all samples from a packet. */
	void unpack_packet(uint8_t *packet, Q16 *samples, Q16 *lpc, uint32_t *sbits) {

		uint32_t rp_bit_ptr=0, n;
		uint32_t m;

		#ifdef PROFILE
			NRF_GPIO->OUTSET = (UNPACK_PIN);
		#endif

		/* Side info first. */
		(*sbits) = packet[0];
		rp_bit_ptr+=8;
		for(n=0;n<P;n++) {
			lpc[n] = 0L;
			lpc[n] |= (packet[rp_bit_ptr/8]<<24);
			rp_bit_ptr+=8;
			lpc[n] |= (packet[rp_bit_ptr/8]<<16);
			rp_bit_ptr+=8;
			lpc[n] |= (packet[rp_bit_ptr/8]<<8);
			rp_bit_ptr+=8;
			lpc[n] |= (packet[rp_bit_ptr/8]);
			rp_bit_ptr+=8;
		}

		/* Then samples. */
		for(n=0;n<N;n++) {

			/* Sign bit first. */
			if (IS_BIT_SET_WITHIN_BYTE(packet[rp_bit_ptr/8], rp_bit_ptr)) {
				samples[n] = 0xFFFF0000L;
			} else {
				samples[n] = 0;
			}
			rp_bit_ptr++;

			/* Then the remaining DBITS-1 bits. */
			for(m=0; m<DBITS-1; m++) {
				samples[n] |= (IS_BIT_SET_WITHIN_BYTE(packet[rp_bit_ptr/8], rp_bit_ptr)<<15-m);
				rp_bit_ptr++;
			}
		}

		#ifdef PROFILE
			NRF_GPIO->OUTCLR = (UNPACK_PIN);
		#endif
	}

	/* Pack side info in the beginning of the packet. Structure: [8 sbits | Px32 LPCs | sound] */
	void pack_side_info(uint32_t sbits, Q16* lpc) {

		uint32_t n;

		/* sbits (8 bits) */
		building_packet[0] = ((uint8_t) sbits);
		sp_bit_ptr+=8;

		/* P LPC coeffs of size 32 bits. */
		for(n=0; n<P; n++) {
			building_packet[sp_bit_ptr/8] = ((uint8_t) (lpc[n]>>24));
			sp_bit_ptr+=8;
			building_packet[sp_bit_ptr/8] = ((uint8_t) (lpc[n]>>16));
			sp_bit_ptr+=8;
			building_packet[sp_bit_ptr/8] = ((uint8_t) (lpc[n]>>8));
			sp_bit_ptr+=8;
			building_packet[sp_bit_ptr/8] = ((uint8_t) lpc[n]);
			sp_bit_ptr+=8;
		}
	}

	/* Add a single sample to packet and updates sp_bit_ptr. */
	void pack_sample(Q16 sample) {

		uint32_t n;

		/* Sign bit first. */
		building_packet[sp_bit_ptr/8] |= SET_BOOL_WITHIN_BYTE((sample<0), sp_bit_ptr);
		sp_bit_ptr++;

		/* Then the remaining DBITS-1 bits. */
		for(n=0; n<DBITS-1; n++) {
			building_packet[sp_bit_ptr/8] |= SET_BOOL_WITHIN_BYTE(IS_BIT_SET(sample, 15-n), sp_bit_ptr);
			sp_bit_ptr++;
		}

		/* If packet is full, move to queue and start a new one. */
		if (sp_bit_ptr>=(PACKET_SIZE_TOTAL*8-1)) {

			/* Copy to tx_packet() and off it goes. */
			for(n=0;n<PACKET_SIZE_TOTAL;n++) {
				tx_packet[n] = building_packet[n];
			}

			transmit();

			/* Reset building packet. */
			memset(building_packet, 0x00, PACKET_SIZE_TOTAL);
			sp_bit_ptr = 0;
		}
	}

	// Assert counter (to be removed)
	//lint -save -esym(551,assert_count) assert_count has no effect.
	static volatile uint32_t assert_count = 0; ///< Assert counter.
	void assert_nrf_callback(uint16_t line_num, const uint8_t *file_name)
	{ assert_count++; }

#endif
