
/*
	Entry point.
	2012 Even brenden
*/

#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include "nrf.h"
#include "uart.h"
#include "Q16.h"
#include "adc.h"
#include "dac.h"
#include "fir_filter.h"
#include "adpcm.h"
#include "midrise.h"
#include "maths.h"
#include "gazell.h"
#include "rx.h"
#include "tx.h"
#include "config.h"

// #define PRINT

/* Encodes block and then ships results to RF immediately. */
void encode_sample(Q16 *sample, Q16 *lpc, int32_t *codeword) {

	/* Encode. */
	#ifdef ADPCM
		adpcm_encode_Q16(sample, &lpc[0], codeword);
	#else
		dpcm_encode_Q16(sample, &lpc[0], codeword);
	#endif

	#ifdef PRINT
		uart_put32(*codeword);
	#endif

	/* Append sample to current packet and let the RF part do the rest. Moved outside of the coding loop for profiling - optimally it's done inline with coding (if N>PACKET_SIZE_BYTES). */
	pack_codeword(codeword);
}

/* Decoder. */
void decode_codeword(Q16 *codeword, Q16 *lpc, Q16 *decoded) {

	/* Decode. */
	#ifdef ADPCM
		adpcm_decode_Q16(codeword, &lpc[0], decoded);
	#else
		dpcm_decode_Q16(codeword, &lpc[0], decoded);;
	#endif
}

int main(void) {

	/* Declares. */
	Q16 sample_to_be_encoded;
	Q16 decoded_sample;
	Q16 codewords_tx[N];
	Q16 codewords_rx[N];
	Q16 lpc_tx[P];
	Q16 lpc_rx[P];
	
	uint32_t rx_samples_ready=0;
	uint32_t n=0;

	/* Nonadaptive LPC. */
	lpc_tx[0] = LPC0;
	lpc_rx[0] = LPC0;

	/* Start up (A)DPCM, UART, ADC and DAC. */
	uart_config();

	#ifdef ADPCM
		adpcm_init();
	#else
		dpcm_init();
	#endif

	#ifdef USE_GZLL
		gazell_init();
		adc_init();
		dac_init();
	#elif defined(IS_TX)
		adc_init();
		tx_init();
	#elif defined(IS_RX)
		dac_init();
		rx_init();
	#else
 		adc_init();
		dac_init();
	#endif

	#if defined(PROFILE) || defined(PROF_ADDA)
		/* Set PINS 0-7 as inputs, 8-15 as outputs for profiling. */
		NRF_GPIO->DIRSET = 0xFF00UL;
		NRF_GPIO->DIRCLR = 0xFFUL;
	#endif

	while(true) {

		#if defined(IS_RX) || defined(USE_GZLL)

			/* Fetch and unpack packet if ready. */
			if ((rx_next_packet_is_ready()) && !rx_samples_ready) {

				#ifdef PROFILE
					NRF_GPIO->OUTSET = (UNPACK_PIN);
				#endif

				buffer_up_rx_packet();
				fetch_new_codewords(&codewords_rx[0]);

				#ifdef PROFILE
					NRF_GPIO->OUTCLR = (UNPACK_PIN);
				#endif
				
				rx_samples_ready=N;
			}

			/* Decode if ready. */			
			if (rx_samples_ready) {
				#ifdef PROFILE
					NRF_GPIO->OUTSET = (DECODE_PIN);
				#endif

				decode_codeword(&codewords_rx[N-rx_samples_ready], &lpc_rx[0], &decoded_sample);

				#ifdef PROFILE
					NRF_GPIO->OUTCLR = (DECODE_PIN);
				#endif

				#ifdef PRINT
					for(n=0;n<N;n++) uart_put32(codewords_rx[N-rx_samples_ready]);
				#endif

				/* Dump block of N samples to DAC. */
				sample_to_dac(&decoded_sample);
				
				rx_samples_ready--;

				#ifdef PRINT
					uart_put32(0xFFFFFFFFUL);
				#endif
			}

		#endif

		#if defined(IS_TX) || defined(USE_GZLL)

			/* Encode if a new sample is ready from the ADC. */
			if (adc_next_sample_is_ready()) {

				#ifdef PROFILE
					NRF_GPIO->OUTSET = (ENCODE_PIN);
				#endif

 				/* Get new sample from ADC. */
				adc_fetch_next_sample(&sample_to_be_encoded);

				/* Encode and send. */
				encode_sample(&sample_to_be_encoded, &lpc_tx[0], &codewords_tx[0]);

				#ifdef NO_RF
					if(tx_next_packet_is_ready()) set_rx_packet(get_tx_packet());
 				#endif

				#ifdef PROFILE
					NRF_GPIO->OUTCLR = (ENCODE_PIN);
				#endif
			}
		#endif

		#ifdef PURE_PWM

			/* Remember to set N=1 before compiling with PURE_PWM set. */
			if (adc_next_sample_is_ready()) {
				adc_fetch_next_sample(&sample_to_be_encoded);
 				block_to_dac(&sample_to_be_encoded);
			}

		#endif
	}
}
