
/*
	Custom math operations such as calculation of one-sided autocorrelation function.
	2011 Even Brenden
*/

#include "maths.h"

/* Logarithm of base 2 for positive Q16 integer numbers. */
uint32_t log_2_Q16(Q16 a) {

	uint32_t temp;

	if(a == 0) {
		return 0;
	}

	temp = 0;

	if(a >= Q16_256)   { a >>=  8; temp +=  8; }
	if(a >= Q16_16)    { a >>=  4; temp +=  4; }
	if(a >= Q16_4)     { a >>=  2; temp +=  2; }
	if(a >= Q16_2)     {           temp +=  1; }

	return temp;
}

Q16 average_Q16(Q16 *x, uint32_t count) {

	Q16 sum = 0;

	uint16_t n;
	for(n=0;n<count;n++) {
		sum+=x[n];
	}
	return DIVQ16(sum, MAKEQ16(0, count, 0));
}

int32_t modulo(int32_t n, int32_t m) {

	if(n>0) {
		for(;n>m;n-=m);
		return n;
	}
	if (n<0) {
		for(;n<0;n+=m);
		return n;
	}
	return n;
}

/* Calculate autocorrelation function. */
void acf_Q16(Q16 *x, Q16 *rxx) {

	uint32_t n, j;

	for(j=0;j<(P+1);j++) {
		rxx[j] = Q16_0;
		for(n=0;n<N;n++) {
			if (n+j<N) {
				rxx[j]+=MULQ16(x[n], x[n+j]);
			}
		}
	}
}

void find_max_Q16(Q16 *x, Q16 *result) {

	Q16 temp;
    uint16_t n;

	*result = Q16_0;

    for(n=0;n<N;n++) {
		temp=absolute_Q16(x[n]);
		if (temp>(*result)) {
			(*result)=temp;
		}
    }
}

/* Get the dynamic range of a block of samples. */
Q16 get_scale_Q16(Q16 *x, uint16_t count) {

	Q16 max=0, temp, min_resolution;
        int n;

        for(n=0;n<N;n++) {
		temp=absolute_Q16(x[n]);
		if (temp>max) {
			max=temp;
		}
        }

	/* To prevent divison by zero. No point in scaling below DBITS resolution. This implies that the step size in the quantizer will underflow and should then be set to LSB (see midrise.c). */
	min_resolution = (1<<(DBITS-1)); // Equivalent to MAKEQ16(0, 0, (1<<(DBITS-1))), only faster.
	max = (max<min_resolution ? max=min_resolution : max);

	if (max==0) {
		return Q16_1; // Doesn't really matter since all values are zero. But an optimization would be to detect a zero scale in the Q and skip the actual Qing.
	} else {
		return DIVQ16(Q16_1, max);
	}
}

int32_t minimum(int32_t v1, int32_t v2) {

	if (v1<v2) {
		return v1;
	} else {
		return v2;
	}
}

int32_t maximum(int32_t v1, int32_t v2) {

        if (v1>v2) {
                return v1;
        } else {
                return v2;
        }
}

Q16 absolute_Q16(Q16 v1) {

	if (v1<0) {
		return ~(v1-1);
	} else {
		return v1;
	}
}
