
/*
	Custom math operations such as calculation of one-sided autocorrelation function.
	2011 Even Brenden
*/

#include <stdio.h>
#include "maths.h"
#include "config.h"

void debug(int no) { // Most convenient to put here...
	printf("---DEBUG #%d---\n", no);
	fflush(stdout);
}

/* Logarithm of base 2 for positive Q16 numbers. */
uint8_t log_2_Q16(Q16 a) {

	uint8_t temp;

	if(a == 0) {
		return 0;
	}

	temp = 0;

	if(a >= Q16_256)   { a >>=  8; temp +=  8; }
	if(a >= Q16_16)    { a >>=  4; temp +=  4; }
	if(a >= Q16_4)     { a >>=  2; temp +=  2; }
	if(a >= Q16_2)     {           temp +=  1; }

	return temp;
}

/* Logarithm of base 2 for positive floats. */
float log_2(float a) {

	float temp;

	if(a == 0) {
		return 0;
	}

	temp = 0;

	if(a >= 65536) { a /= 65536; temp += 16; }
	if(a >= 256)   { a /= 256;   temp +=  8; }
	if(a >= 16)    { a /= 16;    temp +=  4; }
	if(a >= 4)     { a /= 4;     temp +=  2; }
	if(a >= 2)     {             temp +=  1; }

	return (float) temp;
}

Q16 average_Q16(Q16 *x, uint32_t count) {

	Q16 sum = 0;

	uint16_t n;
	for(n=0;n<count;n++) {
		sum+=x[n];
	}
	return DIVQ16(sum, MAKEQ16(0, count, 0));
}


float average(float *x, uint32_t count) {

	float sum = 0;

	uint16_t n;
	for(n=0;n<count;n++) {
		sum+=x[n];
	}
	return sum/count;
}


int32_t modulo(int32_t n, int32_t m) {

	if(n>0) {
		for(;n>m;n-=m);
		return n;
	}
	if (n<0) {
		for(;n<0;n+=m);
		return n;
	}
	return n;
}

/* Calculates autocorrelation function. */
void acf_Q16(Q16 *x, uint16_t count, uint8_t lag, Q16 *rxx) {

	uint16_t n;
	uint8_t j;
	for(j=0;j<lag;j++) {
		rxx[j] = MAKEQ16(0, 0, 0);
		for(n=0;n<count;n++) {
			if ((n+j>=0) & (n+j<count)) {
				rxx[j]+=MULQ16(x[n], x[n+j]);
			}
		}
	}
}

/* Calculates autocorrelation function. */
void acf(float *x, uint16_t count, uint8_t lag, float *rxx) {

	uint16_t n;
	uint8_t j;
	for(j=0;j<lag;j++) {
		rxx[j] = 0.0;
		for(n=0;n<count;n++) {
			if ((n+j>=0) & (n+j<count)) {
				rxx[j]+=x[n]*x[n+j];
			}
		}
	}
}

/* Get the dynamic range of a block of samples. */
Q16 get_scale_Q16(Q16 *x, uint16_t count) {

	Q16 max=0, temp, min_resolution;
        int n;

        for(n=0;n<N;n++) {
		temp=absolute_Q16(x[n]);
		if (temp>max) {
			max=temp;
		}
        }

	/* To prevent divison by zero. No point in scaling below DBITS resolution. This implies that the step size in the quantizer will underflow and should then be set to LSB (see midrise.c). */
	min_resolution = (1<<(DBITS-1)); // Equivalent to MAKEQ16(0, 0, (1<<(DBITS-1))), only faster.
	max = (max<min_resolution ? max=min_resolution : max);

	if (max==0) {
		return Q16_1; // Doesn't really matter since all values are zero. But an optimization would be to detect a zero scale in the Q and skip the actual Qing.
	} else {
		return DIVQ16(Q16_1, max);
	}
}

/* Get the dynamic range of a block of samples. */
float get_scale(float *x, uint16_t count) {


	float max=0, temp;
        uint16_t n;

        for(n=0;n<count;n++) {
		temp=absolute(x[n]);
		if (temp>max) {
			max=temp;
		}
        }

	if(max>0) {
		return 1.0/max;
	} else {
		return 1.0;
	}
}

int32_t minimum(int32_t v1, int32_t v2) {

	if (v1<v2) {
		return v1;
	} else {
		return v2;
	}
}

int32_t maximum(int32_t v1, int32_t v2) {

        if (v1>v2) {
                return v1;
        } else {
                return v2;
        }
}

Q16 absolute_Q16(Q16 v1) {

	if (v1<0) {
		return v1*=-1;
	} else {
		return v1;
	}
}
float absolute(float v1) {

	if (v1<0) {
		return v1*=-1;
	} else {
		return v1;
	}
}
