
/*
	System simulation.
	2011 Even Brenden
*/

#include <stdio.h>
#include <stdint.h>
#include <time.h>
#include <math.h>
#include "maths.h"
#include "l-d.h"
#include "midrise.h"
#include "adpcm.h"
#include "expander.h"
#include "Q16.h"
#include "coding.h"
#include "fir_filter.h"
#include "config.h"

// #define PRINT
// #define PRINT_SAMPLES

void system_Q16() {

	printf("\n----\nSYSTEM SIMULATION WITH Q16\n----\n");
	printf("\n# of LPCs:\t%d", P);
	printf("\nblock size:\t%d", N);
	printf("\nDBITS:\t%d\n", DBITS);

	/* Open file for reading and find size. */

	int n, e;

	FILE *file_in;
	file_in = fopen(FILENAME_IN, "rb");

	if(file_in==0) {
		printf("\nFile does not exist.\n");
	}

	printf("\ninput:\t\t");
	printf(FILENAME_IN);

	fseek(file_in, 0L, SEEK_END);
	int input_size = ftell(file_in);
	input_size /= sizeof(INPUT_SAMPLE_FORMAT);
	int size = input_size;
	fseek(file_in, 0L, SEEK_SET);

	printf("\nfile size:\t%d", size);

	/* If N does not divide size, pad input array so that it does. */
	if (size%N>0)
		size=size+(N-size%N);

	/* Put file contents in buffer and convert to Q16. */
	INPUT_SAMPLE_FORMAT input[size];
	for(n=0;n<size;n++) // Let's start fresh.
		input[n]=0;

	fread(input, sizeof(INPUT_SAMPLE_FORMAT), size, file_in);
	fclose(file_in);

	Q16 input_Q16[size];
	for(n=0; n<size; n++) {
		input_Q16[n] = int16_t2Q16(input[n]);
		#ifdef PRINT_SAMPLES
			PRINTQ16(input_Q16[n]);
		#endif
	}

	/*	   		*/
	/* Start processing. 	*/
	/*			*/

	/* NR init. Find treshold for noise removal. Add margin of 1/32 of dynamic range. Noise is calculated from first block of input. */
	Q16 tresh = average_Q16(&input_Q16[0], N)+MAKEQ16(0, 0, (1<<11));
	Q16 attenuation = MAKEQ16(0, 0, (1<<15));
	uint8_t F=2;

	#ifdef PRINT
	printf("\nnoise tresh:\t");
	PRINTQ16(tresh);
	#endif

	reset_expander(F);


	/* Compression initialization. */
	int32_t codewords[size]; // For ADPCM, these are merely truncated sample values.
	Q16 decoded[size];
	Q16 *lpc = malloc(sizeof(Q16));
	(*lpc) = A_1;
	Q16 error=0;

	#ifdef ADPCM
		init_aqb_coder_Q16();
	#else
		init_coder_Q16();
	#endif


	/* NR, encoding and decoding, block by block. */
	for(n=0;n<size;n+=N) {

		#ifdef PRINT
		printf("\nBLOCK #%d\n", n/N);
		#endif

		/* NR. */
		#ifdef NR
		expander_Q16(&input_Q16[n], N, tresh, attenuation);
		#endif

		/* Comp. */
		#ifdef COMP
			#ifdef ADPCM
				adaptive_encode_Q16(&input_Q16[n], lpc, &codewords[n]);
			#else
				encode_Q16(&input_Q16[n], lpc, &codewords[n]);
			#endif

			/* Fictive channel here. */

			#ifdef ADPCM
				adaptive_decode_Q16(&codewords[n], lpc, &decoded[n]);
			#else
				decode_Q16(&codewords[n], lpc, &decoded[n]);
			#endif

		#else
			for(e=0;e<N;e++) {
				if (n+e<size) {
					decoded[n+e]=input_Q16[n+e];
				}
			}
		#endif

		/* Calculate/append to error */
		for(e=0;e<N;e++) {
			if (n+e<size) {
				error+=MULQ16(input[n+e]-decoded[n+e], input[n+e]-decoded[n+e]);
			}
		}

		/* Pause to analyze */
		#ifdef PRINT
			getchar();
		#endif
	}

	double derror = 1000*(((double) (error>>16))/size);
	printf("\napprox. MSE:\t%fe-3", derror);

	/*						*/
	/* Processing finished.	*/
	/*						*/

	/* Convert result from Q16 to int16_t. Truncated overhead from framing by using input_size instead of size. */
	INPUT_SAMPLE_FORMAT output[input_size];
	for (n=0; n<input_size; n++) {
		output[n]=Q162int16_t(decoded[n]);
		#ifdef PRINT_SAMPLES
		PRINTQ16(decoded[n]);
		#endif
	}

	/* Open and write to new file. */
	FILE *file_out;

	file_out = fopen(FILENAME_OUT, "wb+");
	if (file_out==0) {
		printf("Could not open output file.\n");
	}
	fwrite(output, sizeof(INPUT_SAMPLE_FORMAT), input_size, file_out);
	fclose(file_out);

	printf("\noutput:\t\t");
	printf(FILENAME_OUT);
	printf("\n\n");

	/* Log error. */
	FILE *file_log;
	file_log = fopen(FILENAME_LOG, "a");
	if (file_log==0) {
		printf("Could not open log file.\n");
	}
	time_t cur_time = time(NULL);
	char * time_string = ctime(&cur_time);
	fwrite(time_string, sizeof(char), 24, file_log);
	fwrite("\t", sizeof(char), 1, file_log);
	#ifdef COMP
		#ifdef SSM_AQB
			fwrite("SSM\t", sizeof(char), 4, file_log);
		#elif defined(ADPCM)
			fwrite("ADPCM\t", sizeof(char), 6, file_log);
		#else
			fwrite("DPCM\t", sizeof(char), 5, file_log);
		#endif
		char dbits_str[100];
		int dbits_count = sprintf(dbits_str, "%d", DBITS);
		fwrite("DBITS=", sizeof(char), 6, file_log);
		fwrite(dbits_str, sizeof(char), dbits_count, file_log);
		fwrite("\t", sizeof(char), 1, file_log);
	#endif
	#ifdef NR
		fwrite("NR\t", sizeof(char), 3, file_log);
	#endif
	char n_str[100];
	int n_count = sprintf(n_str, "%d", N);
	fwrite("N=", sizeof(char), 2, file_log);
	fwrite(n_str, sizeof(char), n_count, file_log);
	fwrite("\t", sizeof(char), 1, file_log);

	char error_string[100];
	int error_string_count = sprintf(error_string, "%f", derror);
	fwrite("MSE=", sizeof(char), 4, file_log);
	fwrite(error_string, sizeof(char), error_string_count, file_log);
	fwrite("e-3\n", sizeof(char), 4, file_log);
	fclose(file_log);
}

int main() {

	system_Q16();
	return 0;
}